<?php 
/*
Plugin Name: Blue Hat CDN
Plugin URI: http://www.BlueHatNetwork.com
Description: Plugin for turbo charging website.
Author: Blue Hat Network
Author URI: http://www.BlueHatNetwork.com
Text Domain: blue-hat-cdn
Version: 1.3
*/

$blueHatCDNDBVersion = '1.3';

function blueHatCDNAdminMenu()
{
    add_menu_page('Blue Hat CDN', 'Blue Hat CDN', 'manage_options', 'blue-hat-cdn-mainpage', 'blueHatCDNMainpage', plugins_url('assets/images/icons/bluehatcdn-icon-16.png', __FILE__));
}

function blueHatCDNAdminHead()
{
	wp_enqueue_script('jquery');
	
	wp_enqueue_script('jquery-ui-1.10.3.custom', plugins_url('/assets/js/jquery-ui-1.10.3.custom.min.js', __FILE__), array('jquery'));
	wp_enqueue_script('jquery.jqGrid', plugins_url('/assets/jquery.jqGrid-4.5.4/js/jquery.jqGrid.min.js', __FILE__), array('jquery-ui-1.10.3.custom'));
	wp_enqueue_script('grid.locale', plugins_url('/assets/jquery.jqGrid-4.5.4/js/i18n/grid.locale-'.substr(get_bloginfo('language'), 0, 2).'.js', __FILE__), array('jquery.jqGrid'));
	wp_enqueue_script('jquery.jgrowl', plugins_url('/assets/jGrowl-master/jquery.jgrowl.min.js', __FILE__), array('grid.locale'));
	wp_enqueue_script('bhn', plugins_url('/assets/js/bhn.js', __FILE__), array('jquery.jgrowl'));
	
	wp_enqueue_style('jquery-ui-1.10.3.custom', plugins_url('/assets/css/redmond/jquery-ui-1.10.3.custom.min.css', __FILE__));
	wp_enqueue_style('ui.jqgrid', plugins_url('/assets/jquery.jqGrid-4.5.4/css/ui.jqgrid.css', __FILE__), array('jquery-ui-1.10.3.custom'));
	wp_enqueue_style('jquery.jgrowl', plugins_url('/assets/jGrowl-master/jquery.jgrowl.min.css', __FILE__), array('ui.jqgrid'));
	wp_enqueue_style('styles', plugins_url('/assets/css/styles.css', __FILE__), array('jquery.jgrowl'));
}

function ensureOutputBuffering()
{
	$configLocation = ABSPATH.'wp-config.php';
	
	if(is_writable($configLocation))
	{
		$configContents = file_get_contents($configLocation);
		$configContents = rtrim(preg_replace('@[\r\n\t\s]*?\?>.*?$@sm', '', $configContents)); // ensure there is no closing php bracket at the end of file
		
		$pluginPath = str_replace(ABSPATH, '', plugin_dir_path(__FILE__));
		$configAppendPHP = "\r\n";
		$configAppendPHP .= "/** Begin Blue Hat Turbo Section **/\r\n";
		$configAppendPHP .= "if(file_exists(ABSPATH.'".$pluginPath."buffering.php') && !defined('BLUE_HAT_TURBO_OUTPUT_BUFFERING_STARTED')) include ABSPATH.'".$pluginPath."buffering.php';\r\n";
		$configAppendPHP .= "/** End Blue Hat Turbo Section **/";
		
		if(strpos($configContents, '/** Begin Blue Hat Turbo Section **/') === FALSE)
		{
			$configContents .= $configAppendPHP;
			
			file_put_contents($configLocation, $configContents);
		}
		elseif(strpos($configContents, $pluginPath.'buffering.php') === FALSE)
		{
			// Plugin path has changed, update it
			$configContents = preg_replace('@/\*\* Begin Blue Hat Turbo Section \*\*/.*?/\*\* End Blue Hat Turbo Section \*\*/@ms', '', $configContents);
			
			$configContents .= $configAppendPHP;
			
			file_put_contents($configLocation, $configContents);
		}
	}
	else
	{
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		
		BlueHatNetworkSetting::set('display_admin_message', '<span style="color: red; font-size: 1.2em;">Error: Could not write to config file at '.$configLocation.'. Please ensure the file is writable by the web server, and then refresh this page.</span>');
	}
}

function blueHatCDNMainpage()
{
	ensureOutputBuffering();
	
	if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
	
	if(BlueHatNetworkSetting::get('cache_last_cleared') < 1)
	{
		// Clear initial cache
		if(!class_exists('BlueHatCDNModelBlueHatCDN')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';
		
		$model = new BlueHatCDNModelBlueHatCDN();
		$model->clearWebsiteCache();
	}
	
	include dirname(__FILE__).DIRECTORY_SEPARATOR.'mainpage.php';
}

function blueHatCDNCheckRequirements()
{
	$meetsRequirements = true;

	if(!function_exists('curl_version'))
	{
		$meetsRequirements = false;
		
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		
		BlueHatNetworkSetting::set('display_admin_message', '<span style="color: red; font-size: 1.2em;">Error: '.__('BHN_CURL_NOT_INSTALLED', 'blue-hat-cdn').'</span>');
	}

	return $meetsRequirements;
}

function blueHatCDNDBCheck()
{
    global $blueHatCDNDBVersion;
	
	load_plugin_textdomain('blue-hat-cdn', false, dirname(plugin_basename( __FILE__ )).DIRECTORY_SEPARATOR.'languages'.DIRECTORY_SEPARATOR);
	
    if(get_option('blue_hat_cdn_db_version') != $blueHatCDNDBVersion) blueHatCDNDBInstall();
	
	blueHatCDNCheckRequirements();
}

function blueHatCDNDBInstall()
{
    global $wpdb, $blueHatCDNDBVersion;
    
	require_once ABSPATH.'wp-admin/includes/upgrade.php';
	
	$sql = 'CREATE TABLE IF NOT EXISTS '.$wpdb->prefix.'bhn_file (
			file_id int(11) NOT NULL AUTO_INCREMENT,
			file_full_path varchar(255) COLLATE utf8_bin NOT NULL,
			file_name varchar(255) COLLATE utf8_bin NOT NULL,
			file_path varchar(255) COLLATE utf8_bin NOT NULL,
			file_extension varchar(5) COLLATE utf8_bin NOT NULL,
			file_original_md5 char(32) COLLATE utf8_bin NOT NULL,
			file_original_filesize int(10) unsigned NOT NULL,
			file_final_md5 char(32) COLLATE utf8_bin DEFAULT NULL,
			file_final_filesize int(10) unsigned DEFAULT NULL,
			file_cdate int(10) unsigned NOT NULL,
			file_mdate int(10) unsigned NOT NULL,
			PRIMARY KEY (file_id),
			UNIQUE KEY file_full_path_UNIQUE (file_full_path),
			KEY file_final_md5_idx (file_final_md5),
			KEY file_name_md5 (file_name,file_final_md5)
		  ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;';
    dbDelta($sql);
	
    $sql = 'CREATE TABLE IF NOT EXISTS '.$wpdb->prefix.'bhn_setting (
			setting_id int(10) unsigned NOT NULL AUTO_INCREMENT,
			setting_name varchar(45) COLLATE utf8_bin NOT NULL,
			setting_value varchar(255) COLLATE utf8_bin DEFAULT NULL,
			setting_cdate int(10) unsigned NOT NULL,
			setting_mdate int(10) unsigned NOT NULL,
			PRIMARY KEY (setting_id),
			UNIQUE KEY setting_name_UNIQUE (setting_name)
		  ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;';
	dbDelta($sql);
	
	update_option('blue_hat_cdn_db_version', $blueHatCDNDBVersion);
	
	ensureOutputBuffering();
	
	if(!class_exists('BlueHatCDNModelBlueHatCDN')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';
	
	$model = new BlueHatCDNModelBlueHatCDN();
	$model->clearWebsiteCache();
}

require dirname(__FILE__).DIRECTORY_SEPARATOR.'controller.php';

add_action('wp_ajax_bhn_save_settings', 'bhn_save_settings');
add_action('wp_ajax_bhn_scan_sync_files', 'bhn_scan_sync_files');
add_action('wp_ajax_bhn_get_scan_sync_status', 'bhn_get_scan_sync_status');
add_action('wp_ajax_bhn_get_files', 'bhn_get_files');
add_action('wp_ajax_bhn_clear_index_data', 'bhn_clear_index_data');
add_action('wp_ajax_nopriv_bhn_optimize_database', 'bhn_optimize_database');
add_action('wp_ajax_nopriv_bhn_auto_scan_sync', 'bhn_auto_scan_sync');

add_action('plugins_loaded', 'blueHatCDNDBCheck');

register_activation_hook(__FILE__, 'blueHatCDNDBInstall');

add_action('admin_menu', 'blueHatCDNAdminMenu');
add_action('admin_head', 'blueHatCDNAdminHead');