<?php
/**
 * @package Blue Hat CDN
 * @version 1.3
 * @copyright (C) Copyright 2013 Blue Hat Network, BlueHatNetwork.com. All rights reserved.
 * @license GNU/GPL http://www.gnu.org/licenses/gpl-3.0.txt

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

function bhn_save_settings()
{
	if(current_user_can('manage_options')) 
	{
		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'request.php';
		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';

		$model = new BlueHatCDNModelBlueHatCDN();
		
		$isError = false;
		$errorMsg = '';

		if(!BlueHatNetworkRequest::getVar('cdn_provider')) 
		{
			$isError = true;
			$errorMsg .= __('BHN_SELECT_CDN_ERROR', 'blue-hat-cdn').' ';
		}
		else
		{
			// Check if CDN provider changed
			if(BlueHatNetworkSetting::get('cdn_provider') != BlueHatNetworkRequest::getVar('cdn_provider')) $model->invalidateAllFiles();

			BlueHatNetworkSetting::set('cdn_provider', BlueHatNetworkRequest::getVar('cdn_provider'));
		}

		if(!BlueHatNetworkRequest::getVar('cdn_username') || !BlueHatNetworkRequest::getVar('cdn_api_key')) 
		{
			$isError = true;
			$errorMsg .= __('BHN_PROVIDE_VALID_API_CREDENTIALS', 'blue-hat-cdn').' ';

			BlueHatNetworkSetting::set('cdn_api_key', null);
			BlueHatNetworkSetting::set('cdn_data', null);
			BlueHatNetworkSetting::set('bucket_public_secure_uri', null);
			BlueHatNetworkSetting::set('bucket_public_uri', null);
		}

		if(BlueHatNetworkRequest::getVar('cdn_username'))
		{
			BlueHatNetworkSetting::set('cdn_username', BlueHatNetworkRequest::getVar('cdn_username', null, 'method', 'string', JREQUEST_ALLOWRAW));
		}

		if(!$isError)
		{
			// Validate API Key
			$validateAPICredentials = $model->validateCDNAPICredentials(BlueHatNetworkRequest::getVar('cdn_provider'), BlueHatNetworkRequest::getVar('cdn_username'), BlueHatNetworkRequest::getVar('cdn_api_key'));

			if(!$validateAPICredentials)
			{
				$isError = true;
				$errorMsg .= __('BHN_COULD_NOT_VALIDATE_API_CREDENTIALS_ERROR', 'blue-hat-cdn').' ';

				BlueHatNetworkSetting::set('bucket_public_secure_uri', null);
				BlueHatNetworkSetting::set('bucket_public_uri', null);
				BlueHatNetworkSetting::set('cdn_api_key', null);
				BlueHatNetworkSetting::set('cdn_data', null);
			}
			else
			{
				// Save CDN result data
				BlueHatNetworkSetting::set('bucket_public_secure_uri', $validateAPICredentials['bucket_public_secure_uri']);
				BlueHatNetworkSetting::set('bucket_public_uri', $validateAPICredentials['bucket_public_uri']);
				BlueHatNetworkSetting::set('cdn_api_key', BlueHatNetworkRequest::getVar('cdn_api_key'));
				BlueHatNetworkSetting::set('cdn_data', BlueHatNetworkSerializer::encode($validateAPICredentials));
			}
		}

		$excludeFileList = preg_replace('@[\'"]+@', '', BlueHatNetworkRequest::getVar('exclude_file_list', '', 'method'));
		if(empty($excludeFileList)) $excludeFileList = 'tmp/,logs/,cache/';

		BlueHatNetworkSetting::set('offload_on_fly', BlueHatNetworkRequest::getInt('offload_on_fly', 0, 'method'));
		BlueHatNetworkSetting::set('prefetch_dns', BlueHatNetworkRequest::getInt('prefetch_dns', 0, 'method'));
		BlueHatNetworkSetting::set('optimize_html', BlueHatNetworkRequest::getInt('optimize_html', 0, 'method'));
		BlueHatNetworkSetting::set('include_js', BlueHatNetworkRequest::getInt('include_js', 0, 'method'));
		BlueHatNetworkSetting::set('include_css', BlueHatNetworkRequest::getInt('include_css', 0, 'method'));
		BlueHatNetworkSetting::set('include_images', BlueHatNetworkRequest::getInt('include_images', 0, 'method'));
		BlueHatNetworkSetting::set('optimize_database', BlueHatNetworkRequest::getVar('optimize_database', null, 'method'));
		BlueHatNetworkSetting::set('auto_scan_sync_interval', BlueHatNetworkRequest::getInt('auto_scan_sync_interval', 0, 'method'));
		BlueHatNetworkSetting::set('include_font', BlueHatNetworkRequest::getInt('include_font', 0, 'method'));
		BlueHatNetworkSetting::set('include_swf', BlueHatNetworkRequest::getInt('include_swf', 0, 'method'));
		BlueHatNetworkSetting::set('exclude_files', $excludeFileList);
		
		if($isError)
		{
			BlueHatNetworkSetting::set('display_admin_message', '<span style="color: red; font-size: 1.2em;">Error: '.$errorMsg.'</span>');
		}
		else
		{
			BlueHatNetworkSetting::set('display_admin_message', __('BHN_SETTINGS_SAVED_SUCCESSFULLY', 'blue-hat-cdn'));
		}
		
		$model->clearWebsiteCache();

		wp_redirect('admin.php?page=blue-hat-cdn-mainpage#settings');
	}
	else
	{
		echo 'Access denied for current user.';
	}
	
	die();
}

function bhn_scan_sync_files()
{
	if(current_user_can('manage_options')) 
	{
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		header('Last-Modified: '.gmdate('D, d M Y H:i:s').'GMT');
		header('Cache-Control: no-cache, must-revalidate');
		header('Pragma: no-cache');
		header('Content-type: application/json');

		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';

		$model = new BlueHatCDNModelBlueHatCDN();

		echo BlueHatNetworkSerializer::encode(array('total_num_of_files' => $model->scanFiles()));
	}
	else
	{
		echo 'Access denied for current user.';
	}
	
	die();
}

function bhn_get_scan_sync_status()
{
	if(current_user_can('manage_options')) 
	{
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		header('Last-Modified: '.gmdate('D, d M Y H:i:s').'GMT');
		header('Cache-Control: no-cache, must-revalidate');
		header('Pragma: no-cache');
		header('Content-type: application/json');

		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';

		$model = new BlueHatCDNModelBlueHatCDN();

		echo BlueHatNetworkSerializer::encode(array('status_msg' => BlueHatNetworkSetting::get('status_msg', ''), 'process_mode' => BlueHatNetworkSetting::get('process_mode', 'scan'), 'file_stats' => $model->getFileStats(true)));
	}
	else
	{
		echo 'Access denied for current user.';
	}
	
	die();
}

function bhn_get_files()
{
	if(current_user_can('manage_options')) 
	{
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		header('Last-Modified: '.gmdate('D, d M Y H:i:s').'GMT');
		header('Cache-Control: no-cache, must-revalidate');
		header('Pragma: no-cache');
		header('Content-type: application/json');

		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';

		$model = new BlueHatCDNModelBlueHatCDN();

		echo BlueHatNetworkSerializer::encode($model->getAllFiles(true));
	}
	else
	{
		echo 'Access denied for current user.';
	}
	
	die();
}

function bhn_clear_index_data()
{
	if(current_user_can('manage_options')) 
	{
		require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';

		$model = new BlueHatCDNModelBlueHatCDN();
		$model->resetIndexData();
		
		wp_redirect('admin.php?page=blue-hat-cdn-mainpage');
	}
	else
	{
		echo 'Access denied for current user.';
	}
	
	die();
}

function bhn_optimize_database()
{
	if(!class_exists('BlueHatCDNModelBlueHatCDN')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';
	if(!class_exists('BlueHatNetworkRequest')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'request.php';
	
	$model = new BlueHatCDNModelBlueHatCDN();
	$model->doOptimizeDatabase(BlueHatNetworkRequest::getVar('k', null, 'get'));
	
	die();
}

function bhn_auto_scan_sync()
{
	if(!class_exists('BlueHatCDNModelBlueHatCDN')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';
	if(!class_exists('BlueHatNetworkRequest')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'request.php';
	
	$model = new BlueHatCDNModelBlueHatCDN();
	$model->doAutoScanSync(BlueHatNetworkRequest::getVar('k', null, 'get'));
	
	die();
}