<?php
/**
 * @package Blue Hat CDN
 * @version 1.3
 * @copyright (C) Copyright 2013 Blue Hat Network, BlueHatNetwork.com. All rights reserved.
 * @license GNU/GPL http://www.gnu.org/licenses/gpl-3.0.txt

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class BlueHatNetworkSetting 
{
	static $_allSettings = array();
	
	function &getSettingObject($useCache=true)
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'database.php';
		
		if(empty(self::$_allSettings) || !$useCache)
		{
			$sql = 'SELECT SQL_CACHE 
						setting_name,
						setting_value 
					FROM '.$wpdb->prefix.'bhn_setting;';
			$tmpResults = BlueHatNetworkDatabase::getResults($sql);
			
			self::$_allSettings = array();
			
			if(!empty($tmpResults))
			{
				foreach($tmpResults as $key => $row)
				{
					self::$_allSettings[$row['setting_name']] = $row['setting_value'];
				}
			}
		}
		
		return self::$_allSettings;
	}
	
	function get($settingName, $defaultValue=null, $useCache=true)
	{
		$allSettings =& self::getSettingObject($useCache);
		
		if(array_key_exists($settingName, $allSettings)) return $allSettings[$settingName];
		
		return $defaultValue;
	}
	
	function set($settingName, $newValue=null)
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'database.php';
		if(!class_exists('BlueHatNetworkDateTime')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'datetime.php';
		
		$newValue = ($newValue === null) ? 'null' : BlueHatNetworkDatabase::quote($newValue);
		$nowTimestamp = BlueHatNetworkDateTime::getCurrentUnixStamp();
		
		$sql = 'INSERT IGNORE INTO '.$wpdb->prefix.'bhn_setting 
				( 
					setting_name,
					setting_value,
					setting_cdate,
					setting_mdate
				) 
				VALUES 
				(
					'.BlueHatNetworkDatabase::quote($settingName).', 
					'.$newValue.', 
					'.$nowTimestamp.',
					'.$nowTimestamp.' 
				) ON DUPLICATE KEY UPDATE 
					setting_value = VALUES(setting_value), 
					setting_mdate = VALUES(setting_mdate);';
		
		return BlueHatNetworkDatabase::query($sql);
	}
}