<?php
/**
 * @package Blue Hat CDN
 * @version 1.3
 * @copyright (C) Copyright 2013 Blue Hat Network, BlueHatNetwork.com. All rights reserved.
 * @license GNU/GPL http://www.gnu.org/licenses/gpl-3.0.txt

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class BlueHatCDNModelBlueHatCDNAmazon 
{
	var $_connObj = null;
	var $_processId = 0;
	var $_syncFileCount = 0;
	var $_fileChecksums = array();
	
	function setProcessId($processId)
	{
		$this->_processId = $processId;
	}

	function validateCDNAPICredentials($cdnProvider, $cdnUsername, $cdnAPIKey)
	{
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('S3')) require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'cloudfront'.DIRECTORY_SEPARATOR.'S3.php';
		
		$bucketName = preg_replace('@[^a-zA-Z0-9]@', '_', preg_replace('@^https?://@i', '', site_url(), 1)).'_bhn';
		
		$this->_connObj = new S3($cdnUsername, $cdnAPIKey, true);
		
		try {
			// First make sure bucket exists
			$acctBuckets = $this->_connObj->listBuckets();
			
			if(!in_array($bucketName, $acctBuckets))
			{
				// Bucket doesnt exist, create it
				$this->_connObj->putBucket($bucketName, S3::ACL_PUBLIC_READ);
				
				BlueHatNetworkSetting::set('bucket_name', $bucketName);
			}
			
			// Now make sure cloudfront distribution exists
			$cfDistributions = $this->_connObj->listDistributions();
			$createDistribution = true;
			
			if(!empty($cfDistributions)) 
			{
				foreach($cfDistributions as $cfDistribution)
				{
					if($cfDistribution['origin'] == $bucketName.'.s3.amazonaws.com') $createDistribution = false;
				}
			}
			
			if($createDistribution) $this->_connObj->createDistribution($bucketName);
			
			$cfDistributions = $this->_connObj->listDistributions();
			$cfDistributionDomain = null;
			$cfDistributionId = null;
			
			if(!empty($cfDistributions)) 
			{
				foreach($cfDistributions as $cfDistribution)
				{
					if($cfDistribution['origin'] == $bucketName.'.s3.amazonaws.com') 
					{
						$cfDistributionDomain = $cfDistribution['domain'];
						$cfDistributionId = $cfDistribution['id'];
					}
				}
			}
			
			if(!empty($cfDistributionDomain))
			{
				BlueHatNetworkSetting::set('bucket_public_uri', '//'.$cfDistributionDomain);
				BlueHatNetworkSetting::set('bucket_public_secure_uri', '//'.$cfDistributionDomain);
				BlueHatNetworkSetting::set('cloudfront_distribution_id', $cfDistributionId);
				
				return array('bucket_public_uri' => '//'.$cfDistributionDomain, 'bucket_public_secure_uri' => '//'.$cfDistributionDomain);
			}
			else
			{
				return false;
			}
		} catch(Exception $e) {
			return false;
		}
	}
	
	function syncFiles($allFiles, $fileChecksums)
	{
		global $wpdb;
		
		$this->_fileChecksums = $fileChecksums;
		$this->_syncFileCount = 0;
		
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkLanguage')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'language.php';
		if(!class_exists('BlueHatNetworkNetwork')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'network.php';
		if(!class_exists('BlueHatNetworkSerializer')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		if(!class_exists('BlueHatNetworkDateTime')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'datetime.php';
		if(!class_exists('Minify_JS_ClosureCompiler')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'minify'.DIRECTORY_SEPARATOR.'ClosureCompiler.php';
		if(!class_exists('JSMin')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'minify'.DIRECTORY_SEPARATOR.'JSMin.php';
		if(!class_exists('CSSmin')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'minify'.DIRECTORY_SEPARATOR.'CSSmin.php';
		if(!class_exists('SmushIt')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'minify'.DIRECTORY_SEPARATOR.'SmushIt.class.php';
		if(!class_exists('S3')) require_once dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'cloudfront'.DIRECTORY_SEPARATOR.'S3.php';
		
		if(!empty($allFiles))
		{
			$siteBaseRoot = (strncasecmp(PHP_OS, 'WIN', 3) == 0) ? str_replace('\\', '/', rtrim(ABSPATH, DIRECTORY_SEPARATOR)) : rtrim(ABSPATH, DIRECTORY_SEPARATOR);
			$cdnProvider = BlueHatNetworkSetting::get('cdn_provider');
			$cdnUsername = BlueHatNetworkSetting::get('cdn_username');
			$cdnAPIKey = BlueHatNetworkSetting::get('cdn_api_key');
			
			// First lets validate CDN credentials make sure they are valid
			if(!$this->validateCDNAPICredentials($cdnProvider, $cdnUsername, $cdnAPIKey))
			{
				BlueHatNetworkSetting::set('status_msg', BlueHatNetworkLanguage::_('BHN_COULD_NOT_VALIDATE_API_CREDENTIALS_ERROR'));
				
				sleep(2); // give it seconds to display status message to user
				
				return false;
			}
			
			$bucketName = BlueHatNetworkSetting::get('bucket_name');
			
			if(!empty($allFiles) && !empty($bucketName))
			{
				$currentUnixTimeStamp = BlueHatNetworkDateTime::getCurrentUnixStamp();
				$cacheExpireTime = 86400*365; // 30 days
				$expiresTime = $currentUnixTimeStamp+$cacheExpireTime;
				$expiresDateTimeFormat = gmdate('D, d M Y H:i:s \G\M\T', $expiresTime);
				$allFilesOriginalChecksums = $this->getAllFilesOriginalChecksums();
				$invalidateFilesArray = array();
				
				foreach($allFiles as $key => $syncFile)
				{
					if(BlueHatNetworkSetting::get('process_id', null, false) != $this->_processId) exit();
					
					$doSyncForThisFile = false;
					
					if(isset($allFilesOriginalChecksums[$syncFile]))
					{
						// File is indexed in DB, check checksum
						if(empty($allFilesOriginalChecksums[$syncFile]['file_final_md5']) || $allFilesOriginalChecksums[$syncFile]['file_original_md5'] != md5_file($siteBaseRoot.$syncFile)) $doSyncForThisFile = true;
					}
					else
					{
						// File doesnt even exist in DB
						$doSyncForThisFile = true;
					}
					
					if($doSyncForThisFile)
					{
						$remotePath = ltrim($syncFile, '/');
						
						$optimizedFilePath = sys_get_temp_dir().DIRECTORY_SEPARATOR.basename($syncFile);

						if(file_exists($optimizedFilePath)) unlink($optimizedFilePath);

						if(!copy($siteBaseRoot.$syncFile, $optimizedFilePath)) continue;

						$optimizationAttemptFailed = false;

						// First attempt optimize filesize
						if(preg_match('@\.js$@i', $syncFile))
						{
							// JS File
							try {
								file_put_contents($optimizedFilePath, $this->removeJSComments(Minify_JS_ClosureCompiler::minify(file_get_contents($siteBaseRoot.$syncFile))));
							} catch(Exception $e) {
								$optimizationAttemptFailed = true;
							}
						}
						elseif(preg_match('@\.css$@i', $syncFile))
						{
							try {
								$cssMinObj = new CSSmin(false);

								file_put_contents($optimizedFilePath, $this->removeJSComments($cssMinObj->run(file_get_contents($siteBaseRoot.$syncFile))));
							} catch(Exception $e) {
								$optimizationAttemptFailed = true;
							}
						}
						elseif(preg_match('@\.(?:png|gif|jpe?g|ico)$@i', $syncFile))
						{
							// Image file lossless optimization
							try {
								$smushit = new SmushIt(array($optimizedFilePath), SmushIt::THROW_EXCEPTION);

								foreach($smushit->get() as $optimizedImg) 
								{
									if(empty($optimizedImg) || !is_object($optimizedImg) || !isset($optimizedImg->destination)) $optimizationAttemptFailed = true;
									if(!$optimizationAttemptFailed) if(!copy($optimizedImg->destination, $optimizedFilePath)) $optimizationAttemptFailed = true;
								}
							} catch(Exception $e) {
								$optimizationAttemptFailed = true;
							}
						}

						clearstatcache();

						if($optimizationAttemptFailed || filesize($optimizedFilePath) < 1 || filesize($optimizedFilePath) > filesize($siteBaseRoot.$syncFile)) copy($siteBaseRoot.$syncFile, $optimizedFilePath);

						$shouldGzip = $this->gzipFile($optimizedFilePath);

						clearstatcache();

						$finalFileSize = filesize($optimizedFilePath);
						$finalFileChecksum = md5_file($optimizedFilePath);

						// Now upload the file to CDN
						$fileUploadWasSuccess = false;

						try {
							$fileCorrectMimeType = $this->getFileMimeType($optimizedFilePath);
							
							$metaHeaders = array();
							$requestHeaders = array(
								'Cache-Control' => 'max-age='.$cacheExpireTime,
								'Expires' => $expiresDateTimeFormat
							);
							
							if(!empty($fileCorrectMimeType)) $requestHeaders['Content-Type'] = $fileCorrectMimeType;
							if($shouldGzip) $requestHeaders['Content-Encoding'] = 'gzip';
							if(preg_match('@\.(?:css|js|swf|svg|ttf|otf|woff|eot)$@i', $syncFile)) $requestHeaders['Access-Control-Allow-Origin'] = '*';
							
							$this->_connObj->putObjectFile($optimizedFilePath, $bucketName, $remotePath, S3::ACL_PUBLIC_READ, $metaHeaders, $requestHeaders);
							
							$fileUploadWasSuccess = true;
						} catch(Exception $e) {
							$fileUploadWasSuccess = false;
						}

						if($fileUploadWasSuccess)
						{
							if($this->_syncFileCount > 1)
							{
								BlueHatNetworkSetting::set('status_msg', sprintf(__('BHN_SYNCING_FILES', 'blue-hat-cdn'), $this->_syncFileCount, $syncFile));
							}
							else
							{
								BlueHatNetworkSetting::set('status_msg', sprintf(__('BHN_SYNCING_FILE', 'blue-hat-cdn'), $this->_syncFileCount, $syncFile));
							}

							$sql = 'UPDATE '.$wpdb->prefix.'bhn_file SET 
										file_final_md5 = '.BlueHatNetworkDatabase::quote($finalFileChecksum).',
										file_final_filesize = '.(int)$finalFileSize.', 
										file_mdate = '.$currentUnixTimeStamp.' 
									WHERE file_full_path = '.BlueHatNetworkDatabase::quote($syncFile).';';
							BlueHatNetworkDatabase::query($sql);

							$this->_syncFileCount++;
							
							$invalidateFilesArray[] = $syncFile;
						}

						unlink($optimizedFilePath); // delete tmp optimized file when done with it
					}
				}
				
				if(!empty($invalidateFilesArray))
				{
					$chunked = array_chunk($invalidateFilesArray, 1000);

					foreach($chunked as $chunkedInvalidateArr)
					{
						try {
							$this->_connObj->invalidateDistribution(BlueHatNetworkSetting::get('cloudfront_distribution_id'), $chunkedInvalidateArr);
						} catch(Exception $e) {

						}
					}
				}
				
				sleep(2);
			}
		}
	}
	
	function getAllFilesOriginalChecksums()
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		
		$result = array();
		
		$sql = 'SELECT file_full_path, file_original_md5, file_final_md5 FROM '.$wpdb->prefix.'bhn_file;';
		
		$allFiles = BlueHatNetworkDatabase::getResults($sql);
		
		if(!empty($allFiles))
		{
			foreach($allFiles as $dbFile)
			{
				$result[$dbFile['file_full_path']] = array('file_original_md5' => $dbFile['file_original_md5'], 'file_final_md5' => $dbFile['file_final_md5']);
			}
		}
		
		return $result;
	}
	
	function getFileMimeType($filename)
	{
		$result = false;
		
		if(preg_match('@\.js$@i', $filename))
		{
			$result = 'text/javascript';
		}
		elseif(preg_match('@\.css$@i', $filename))
		{
			$result = 'text/css';
		}
		elseif(preg_match('@\.swf$@i', $filename))
		{
			$result = 'application/x-shockwave-flash';
		}
		elseif(preg_match('@\.png$@i', $filename))
		{
			$result = 'image/png';
		}
		elseif(preg_match('@\.jpe?g$@i', $filename))
		{
			$result = 'image/jpeg';
		}
		elseif(preg_match('@\.gif$@i', $filename))
		{
			$result = 'image/gif';
		}
		elseif(preg_match('@\.ico$@i', $filename))
		{
			$result = 'image/x-icon';
		}
		elseif(preg_match('@\.eot$@i', $filename))
		{
			$result = 'application/vnd.ms-fontobject';
		}
		elseif(preg_match('@\.otf$@i', $filename))
		{
			$result = 'font/opentype';
		}
		elseif(preg_match('@\.ttf$@i', $filename))
		{
			$result = 'font/ttf';
		}
		elseif(preg_match('@\.woff$@i', $filename))
		{
			$result = 'application/font-woff';
		}
		elseif(preg_match('@\.svg$@i', $filename))
		{
			$result = 'image/svg+xml';
		}
		
		return $result;
	}
	
	function gzipFile($filename)
	{
		$fileContents = file_get_contents($filename);
		
		// Open the gz file (w9 is the highest compression)
		$fp = gzopen($filename.'.gz', 'w9');
		
		// Compress the file
		gzwrite($fp, $fileContents);
		
		// Close the gz file and we are done
		gzclose($fp);
		
		clearstatcache();
		
		// Check if new size is bigger than original
		if(filesize($filename.'.gz') > filesize($filename)) 
		{
			// Restore original ungzipped version
			unlink($filename.'.gz');
			
			return false;
		}
		else
		{
			rename($filename.'.gz', $filename);
			
			return true;
		}
	}
	
	function removeJSComments($txt)
	{
		return $txt;
	}
	
	function getFilesSyncedCount()
	{
		return $this->_syncFileCount;
	}
}