<?php
/**
 * @package Blue Hat CDN
 * @version 1.3
 * @copyright (C) Copyright 2013 Blue Hat Network, BlueHatNetwork.com. All rights reserved.
 * @license GNU/GPL http://www.gnu.org/licenses/gpl-3.0.txt

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class BlueHatCDNModelBlueHatCDNBHN 
{
	var $_apiUri = 'https://api.bluehatnetwork.com/index.php?option=com_bluehatnetwork&view=api';
	var $_processId = 0;
	var $_syncFileCount = 0;
	var $_fileChecksums = array();
	
	function setProcessId($processId)
	{
		$this->_processId = $processId;
	}

	function validateCDNAPICredentials($cdnProvider, $cdnUsername, $cdnAPIKey)
	{
		if(!class_exists('BlueHatNetworkNetwork')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'network.php';
		if(!class_exists('BlueHatNetworkSerializer')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		
		$response = BlueHatNetworkNetwork::post($this->_apiUri.'&task=bucket_info', array(
																							'api_username' => $cdnUsername,
																							'bucket_api_key' => $cdnAPIKey
																						));

		if(!empty($response))
		{
			$response = (array)BlueHatNetworkSerializer::decode($response);
			
			if(isset($response['bucket_path'])) if(!empty($response['bucket_path'])) return $response;
		}
		
		return false;
	}
	
	function syncFiles($allFiles, $fileChecksums)
	{
		global $wpdb;
		
		$this->_fileChecksums = $fileChecksums;
		$this->_syncFileCount = 0;
		
		if(!class_exists('BlueHatNetworkSetting')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkLanguage')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'language.php';
		if(!class_exists('BlueHatNetworkNetwork')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'network.php';
		if(!class_exists('BlueHatNetworkSerializer')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		if(!class_exists('BlueHatNetworkDateTime')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'datetime.php';
		if(!class_exists('Minify_JS_ClosureCompiler')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'minify'.DIRECTORY_SEPARATOR.'ClosureCompiler.php';
		if(!class_exists('JSMin')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'minify'.DIRECTORY_SEPARATOR.'JSMin.php';
		if(!class_exists('CSSmin')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'minify'.DIRECTORY_SEPARATOR.'CSSmin.php';
		if(!class_exists('SmushIt')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'minify'.DIRECTORY_SEPARATOR.'SmushIt.class.php';
		
		if(!empty($allFiles))
		{
			$siteBaseRoot = (strncasecmp(PHP_OS, 'WIN', 3) == 0) ? str_replace('\\', '/', rtrim(ABSPATH, DIRECTORY_SEPARATOR)) : rtrim(ABSPATH, DIRECTORY_SEPARATOR);
			$cdnProvider = BlueHatNetworkSetting::get('cdn_provider');
			$cdnUsername = BlueHatNetworkSetting::get('cdn_username');
			$cdnAPIKey = BlueHatNetworkSetting::get('cdn_api_key');
			
			// First lets validate CDN credentials make sure they are valid
			if(!$this->validateCDNAPICredentials($cdnProvider, $cdnUsername, $cdnAPIKey))
			{
				BlueHatNetworkSetting::set('status_msg', BlueHatNetworkLanguage::_('BHN_COULD_NOT_VALIDATE_API_CREDENTIALS_ERROR'));
				
				sleep(2); // give it seconds to display status message to user
				
				return false;
			}
			
			if(!empty($allFiles))
			{
				$currentUnixTimeStamp = BlueHatNetworkDateTime::getCurrentUnixStamp();
				
				$allFilesOriginalChecksums = $this->getAllFilesOriginalChecksums();
				
				foreach($allFiles as $key => $syncFile)
				{
					if(BlueHatNetworkSetting::get('process_id', null, false) != $this->_processId) exit();
					
					$doSyncForThisFile = false;
					
					if(isset($allFilesOriginalChecksums[$syncFile]))
					{
						// File is indexed in DB, check checksum
						if(empty($allFilesOriginalChecksums[$syncFile]['file_final_md5']) || $allFilesOriginalChecksums[$syncFile]['file_original_md5'] != md5_file($siteBaseRoot.$syncFile)) $doSyncForThisFile = true;
					}
					else
					{
						// File doesnt even exist in DB
						$doSyncForThisFile = true;
					}
					
					if($doSyncForThisFile) 
					{
						$optimizedFilePath = sys_get_temp_dir().DIRECTORY_SEPARATOR.basename($syncFile);

						if(file_exists($optimizedFilePath)) unlink($optimizedFilePath);

						if(!copy($siteBaseRoot.$syncFile, $optimizedFilePath)) continue;

						$optimizationAttemptFailed = false;

						// First attempt optimize filesize
						if(preg_match('@\.js$@i', $syncFile))
						{
							// JS File
							try {
								file_put_contents($optimizedFilePath, $this->removeJSComments(Minify_JS_ClosureCompiler::minify(file_get_contents($siteBaseRoot.$syncFile))));
							} catch(Exception $e) {
								$optimizationAttemptFailed = true;
							}
						}
						elseif(preg_match('@\.css$@i', $syncFile))
						{
							try {
								$cssMinObj = new CSSmin(false);

								file_put_contents($optimizedFilePath, $this->removeJSComments($cssMinObj->run(file_get_contents($siteBaseRoot.$syncFile))));
							} catch(Exception $e) {
								$optimizationAttemptFailed = true;
							}
						}
						elseif(preg_match('@\.(?:png|gif|jpe?g|ico)$@i', $syncFile))
						{
							// Image file lossless optimization
							try {
								$smushit = new SmushIt(array($optimizedFilePath), SmushIt::THROW_EXCEPTION);

								foreach($smushit->get() as $optimizedImg) 
								{
									if(empty($optimizedImg) || !is_object($optimizedImg) || !isset($optimizedImg->destination)) $optimizationAttemptFailed = true;
									if(!$optimizationAttemptFailed) if(!copy($optimizedImg->destination, $optimizedFilePath)) $optimizationAttemptFailed = true;
								}
							} catch(Exception $e) {
								$optimizationAttemptFailed = true;
							}
						}

						clearstatcache();

						if($optimizationAttemptFailed || filesize($optimizedFilePath) < 1 || filesize($optimizedFilePath) > filesize($siteBaseRoot.$syncFile)) copy($siteBaseRoot.$syncFile, $optimizedFilePath);

						$postResult = BlueHatNetworkNetwork::post($this->_apiUri.'&task=put_file', array(
							'api_username' => $cdnUsername,
							'bucket_api_key' => $cdnAPIKey,
							'file_path' => $syncFile,
							'file_name' => '@'.$optimizedFilePath
						), true);
						
						if(!empty($postResult))
						{
							$postResult = (array)BlueHatNetworkSerializer::decode($postResult);

							if((int)@$postResult['success'] == 1)
							{
								// Post was a success
								if($this->_syncFileCount > 1)
								{
									BlueHatNetworkSetting::set('status_msg', sprintf(__('BHN_SYNCING_FILES', 'blue-hat-cdn'), $this->_syncFileCount, $syncFile));
								}
								else
								{
									BlueHatNetworkSetting::set('status_msg', sprintf(__('BHN_SYNCING_FILE', 'blue-hat-cdn'), $this->_syncFileCount, $syncFile));
								}

								$sql = 'UPDATE '.$wpdb->prefix.'bhn_file SET 
											file_final_md5 = '.BlueHatNetworkDatabase::quote($postResult['file_final_checksum']).',
											file_final_filesize = '.(int)$postResult['file_final_filesize'].', 
											file_mdate = '.$currentUnixTimeStamp.' 
										WHERE file_full_path = '.BlueHatNetworkDatabase::quote($syncFile).';';
								BlueHatNetworkDatabase::query($sql);

								$this->_syncFileCount++;
							}
						}

						unlink($optimizedFilePath); // delete tmp optimized file when done with it
					}
				}
				
				BlueHatNetworkNetwork::closeConnection();
				
				sleep(2);
			}
		}
	}
	
	function getAllFilesOriginalChecksums()
	{
		global $wpdb;
		
		if(!class_exists('BlueHatNetworkDatabase')) require dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		
		$result = array();
		
		$sql = 'SELECT file_full_path, file_original_md5, file_final_md5 FROM '.$wpdb->prefix.'bhn_file;';
		
		$allFiles = BlueHatNetworkDatabase::getResults($sql);
		
		if(!empty($allFiles))
		{
			foreach($allFiles as $dbFile)
			{
				$result[$dbFile['file_full_path']] = array('file_original_md5' => $dbFile['file_original_md5'], 'file_final_md5' => $dbFile['file_final_md5']);
			}
		}
		
		return $result;
	}
	
	function removeJSComments($txt)
	{
		return $txt;
	}
	
	function getFilesSyncedCount()
	{
		return $this->_syncFileCount;
	}
}